/*
The Kurz Web UI exposes HTTP routes for browsers, route names to access them, and types for the requests.

These routes are exposed by running SetupUI(listenAddress), which is enough to
configure the Kurz domain API. Be sure to also configure the domain SPI to have
a complete application.
*/
package web

import (
	"errors"
	"fmt"
	"html/template"
	"log"
	"net/http"
	"net/url"
	"path"
	"path/filepath"
	"strconv"
	"strings"

	"github.com/gorilla/mux"
	"github.com/gorilla/sessions"
)

// Route names.
const (
	RouteGetRoot    = "kurz.web.get_root"
	RouteGetShort   = "kurz.web.get_short"
	RoutePostTarget = "kurz.web.post_target"
)

// Content types.
const (
	HtmlFormType = "application/x-www-form-urlencoded"

	// HtmlType is the MIME HTML type.
	HtmlType = "text/html"

	// HtmlTypeRegex is a regex matching the MIME HTML type anywhere
	HtmlTypeRegex = HtmlType
)

type Globals struct {
	AssetsBaseURL string
	AssetsVersion int
	AssetsPath    string
	SessionKey    []byte
	SessionName   string
	SiteBaseURL   string
	RefreshDelay  int
	SiteName      string
}

var globals Globals
var store *sessions.CookieStore
var tmpl *template.Template

// SetupUI() configures Web UI routes on the passed mux.Router.
func SetupUI(router *mux.Router, configAssetsPath string) {
	// Set up asset routes first, for them to have priority over possibly matching short URLs.
	setupAssetRoutes(configAssetsPath, router)
	setupControllerRoutes(router)
	setupTemplates(configAssetsPath)
}

func setupAssetRoutes(configAssetsPath string, router *mux.Router) {
	absAssetsDir, err := filepath.Abs(configAssetsPath)
	if err != nil {
		panic(err)
	}
	log.Printf("Serving assets from %s\n", absAssetsDir)
	fs := http.FileServer(http.Dir(absAssetsDir))
	router.Handle("/favicon.ico", fs)
	for _, prefix := range []string{"css", "js", "images"} {
		router.PathPrefix("/" + prefix).Handler(fs)
	}
}

func setupControllerRoutes(router *mux.Router) {
	// BUG(fgm): improve Accept header matchers once https://github.com/golang/go/issues/19307 is completed.
	router.HandleFunc("/{short}", func(w http.ResponseWriter, r *http.Request) {
		handleGetShort(w, r, router)
	}).
		Methods("GET", "HEAD").
		Name(RouteGetShort)
	router.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
		handlePostTarget(w, r, router)
	}).
		HeadersRegexp("Accept", HtmlTypeRegex).
		Headers("Content-Type", HtmlFormType).
		Methods("POST").
		Name(RoutePostTarget)
	router.HandleFunc("/", handleGetRoot).
		Methods("GET", "HEAD").
		Name(RouteGetRoot)
}

func setupTemplates(configAssetsPath string) {
	base, _ := filepath.Abs(configAssetsPath + "/../templates/")
	layout := base + "/layout"
	funcMap := template.FuncMap{
		"asset": URLForAsset,
	}
	tmpl = template.Must(template.New("kurz").
		Funcs(funcMap).
		ParseFiles(
			base+"/201.gohtml",
			base+"/404.gohtml",
			base+"/409.gohtml",
			base+"/home.gohtml",
			layout+"/analytics.gohtml",
			layout+"/flashes.gohtml",
			layout+"/footer.gohtml",
			layout+"/inlinecss.gohtml",
		))
}

func SetupGlobals(c map[string]interface{}) {
	// Note: keys in viper are lower-cased.
	globals = Globals{
		AssetsBaseURL: strings.Trim(c["assetsbaseurl"].(string), "/"),
		AssetsPath:    c["assetspath"].(string),
		AssetsVersion: c["assetsversion"].(int),
		RefreshDelay:  c["refreshdelay"].(int),
		SessionKey:    []byte(c["sessionkey"].(string)),
		SessionName:   c["sessionname"].(string),
		SiteBaseURL:   strings.Trim(c["sitebaseurl"].(string), "/"),
		SiteName:      c["sitename"].(string),
	}

	store = sessions.NewCookieStore(globals.SessionKey)
}

/*
URLFromRoute generates absolute URLs for named routes.

To build URLs for assets, use URLForAsset().

  - ns: the assets namespace. One of "js", "css', "images".
  - path: the asset path relative to the project root
*/
func URLFromRoute(router *mux.Router, name string, params map[string]string) (string, error) {
	route := router.Get(name)
	if route == nil {
		err := errors.New(fmt.Sprintf("Error building unregistered route %s\n", name))
		return "", err
	}
	pairs := mapToPairs(params)
	url, err := route.URL(pairs...)
	if err != nil {
		err = errors.New(fmt.Sprintf("Error building route %s: %s\n", name, err))
		return "", err
	}
	// Don't use path.Join, it would clobber the scheme://domain format.
	// SiteBaseURL is "/"-trimmed on setup.
	fqsu := globals.SiteBaseURL + url.String()
	return fqsu, nil
}

/*
URLFromRoute generates absolute URLs for assets.

To build URLs for routes, use URLFromRoute().

  - ns: the assets namespace. One of "", "js", "css', "images". "" is only expected
    to be used for "favicon.ico".
  - path: the asset path relative to the project root
*/
func URLForAsset(ns string, assetPath string) (string, error) {
	if ns != "" && ns != "css" && ns != "js" && ns != "images" {
		return "", errors.New("invalid asset namespace: " + ns)
	}
	version := globals.AssetsVersion
	base, err := url.Parse(globals.AssetsBaseURL)
	if err != nil {
		panic(err)
	}
	// Handles "" cleanly, and doesn't use a "\" on windows, unlike filepath.Join.
	base.Path = path.Join(base.Path, ns, assetPath)

	// No need to url.QueryEscape() since this format is query-clean by construction.
	base.RawQuery = "v=" + strconv.Itoa(version)

	res := base.String()
	return res, nil
}
